/*
 * CrudContainer
 * Copyright (C) 2011 by Apaq
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package dk.apaq.framework.criteria;

import dk.apaq.framework.criteria.Rule.FiltrationItem;
import java.util.Arrays;
import java.util.List;

/**
 * A sorter that specifies how sorting of data.
 */
public class Sorter {

    public enum Direction {

        Ascending, Descending
    }

    /**
     * Enum used by Sorter and SorterEntry when sorting items.
     */
    private enum SortResult {
        LessThan, Equal, GreaterThan
    }

    public static class SorterEntry {

        private final String propertyId;
        private final Direction direction;

        /**
         * Creates a new SorterEntry.
         * @param propertyId The id for sorting.
         * @param direction The direction of sorting.
         */
        public SorterEntry(String propertyId, Direction direction) {
            if (propertyId == null) {
                throw new IllegalArgumentException("propertyid cannot be null.");
            }

            if (direction == null) {
                throw new IllegalArgumentException("direction cannot be null.");
            }

            this.propertyId = propertyId;
            this.direction = direction;
        }

        /**
         * Retrieves the direction.
         * @return The direction
         */
        public Direction getDirection() {
            return direction;
        }

        /**
         * Retrieves the propertyid to use for sorting.
         * @return The propertyid.
         */
        public String getPropertyId() {
            return propertyId;
        }

        /**
         * Compares 2 items.
         * @param item1 First item.
         * @param item2 Second item.
         * @return The result for comparing the 2 items.
         */
        public SortResult compare(Rule.FiltrationItem item1, Rule.FiltrationItem item2) {
            Object prop1 = item1.getFiltrationProperty(propertyId);
            Object prop2 = item2.getFiltrationProperty(propertyId);

            if (prop1 instanceof Comparable) {
                int result = ((Comparable) prop1).compareTo(prop2);
                if (result == 0) {
                    return SortResult.Equal;
                }

                if (direction == Direction.Ascending) {
                    return result < 0 ? SortResult.LessThan : SortResult.GreaterThan;
                } else {
                    return result > 0 ? SortResult.LessThan : SortResult.GreaterThan;
                }
            }
            return SortResult.Equal;
        }
    }
    
    public static class Comparator implements java.util.Comparator<Rule.FiltrationItem> {
        
        private final Sorter sorter;

        public Comparator(Sorter sorter) {
            this.sorter = sorter;
        }

        public int compare(FiltrationItem f1, FiltrationItem f2) {
            SortResult result = sorter.compare(f1, f2);
            switch(result) {
                case GreaterThan:
                    return 1;
                case LessThan:
                    return -1;
                default:
                    return 0;
            }
        }
        
        
    }
    
    private final List<SorterEntry> elements;

    /**
     * Creates a new instance with the specified elements.
     * @param elements The sorterentires for this sorter.
     */
    public Sorter(List<SorterEntry> elements) {
        this.elements = elements;
    }

    /**
     * Creates a new instance with the specified elements.
     * @param elements The sorterentires for this sorter.
     */
    public Sorter(SorterEntry... entries) {
        this(Arrays.asList(entries));
    }

    /**
     * Creates a new sorter that sorts on a single property. The sorting
     * direction will be ascending.
     * @param propertyId The propertyId to use for sorting.
     */
    public Sorter(String propertyId) {
        this(propertyId, Direction.Ascending);
    }

    /**
     * Creates a new sorter that sorts on a single property.
     * @param propertyId The propertyId to use for sorting.
     * @param direction The direction for the sorting.
     */
    public Sorter(String propertyId, Direction direction) {
        this(new SorterEntry(propertyId, direction));
    }

    /**
     * Retrieve Sorter entries.
     * @return The sorter entires.
     */
    public List<SorterEntry> getSorterEntries() {
        return elements;
    }

    /**
     * Compare 2 items using this sorter.
     * @param filterItem1 The first item.
     * @param filterItem2 The second item.
     * @return The result.
     */
    public SortResult compare(Rule.FiltrationItem filterItem1, Rule.FiltrationItem filterItem2) {
        for (SorterEntry sorter : elements) {
            SortResult result = sorter.compare(filterItem1, filterItem2);
            if (result != SortResult.Equal) {
                return result;
            }
        }
        return SortResult.Equal;
    }
}
